TREECAT(1)

# NAME

treecat - serialize a directory into a tree diagram, and vice versa

# SYNOPSIS

treecat [-avcC] [-r <file>] files...

# DESCRIPTION

## Options

[[ -a
:[ (all) do not ignore entries starting with '.'
|  -c
:  (colorless) don't use ANSI escape codes to color the output
|  -C
:  (color) use ANSI escape codes to color the output, even when output to a pipe
|  -f
:  (full) print filenames with full path relative to start directory
|  -r <file>
:  (reverse) parse a treecat from <file> to create a filetree (use '-' to read from stdin)
|  -v
:  (verbose) print filenames as they are extracted in reverse mode

## Arguments

The interpretation of treecat's arguments depends on whether it has been
invoked in reverse mode (-r) or not, but in both cases treecat takes zero or
more arguments.

*Normal mode (filesystem => text):*

Treecat will print out a tree for each of its arguments, with the outputs of
different arguments separated by a blank line. If no arguments are given,
treecat will run on the current directory.

*Reverse mode (text => filesystem):*

If no arguments are given, treecat will attempt to parse all of its input,
producing multiple different filetrees if the input contains multiple trees.
The destination of the created filetrees will be taken from the root name of
each tree.

If one or more arguments are given, treecat will attempt to parse one tree from
its input for each argument. The name of the argument will be used as the
destination of the created filetree.

# OUTPUT

Files that are valid UTF-8 will be output as text, and files that are not valid
UTF-8 will be printed out as a hexdump.

A single-character marker is used after each filename to indicate what kind of
file it is. The meanings of these are as follows:

|[ /
:[ directory
|  @
:  symlink
|  \:
:  regular text file
|  \+
:  executable text file
|  #
:  hexdump of a binary file
|  \*
:  hexdump of an executable binary file

A file is considered "executable" if it has the u+x permission. No other file
permissions are taken into account, and any other file types will be ignored.

## Filename escaping

Treecat performs simple backslash-escaping on filenames. The set of special
escape codes are as follows:

|[ \\a
:[ 0x07 (alert)
|  \\b
:  0x08 (backspace)
|  \\e
:  0x1b (escape)
|  \\f
:  0x0c (form feed)
|  \\n
:  0x0a (newline)
|  \\r
:  0x0d (carriage return)
|  \\t
:  0x09 (tab)
|  \\v
:  0x0b (vertical tab)
|  \\\\
:  0x5c (backslash)

The rest of the ASCII control characters are all escaped as \\xHH, where H is a
hex digit.

# EXAMPLES

```
treecat

treecat -r input.treecat destination

curl https://git.sr.ht/~autumnull/treecat/blob/main/test/3 | treecat -r -

treecat old/a old/b old/c | treecat -r - new/a new/b new/c
```

# AUTHORS

Maintained by Autumn! <autumnull@posteo.net>. Up-to-date sources can be found at
https://git.sr.ht/~autumnull/treecat and bugs/patches can be submitted by email
to ~autumnull/treecat-dev@lists.sr.ht
